@@ -1,21 +1,21 @@
 
 /*
- * $Id: stat.c,v 1.311.2.5 1999/09/23 05:55:34 wessels Exp $
+ * $Id: stat.c,v 1.351.2.11 2004/12/08 01:00:56 hno Exp $
  *
  * DEBUG: section 18    Cache Manager Statistics
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ static void statCountersInitSpecial(StatCounters *);
 static void statCountersClean(StatCounters *);
 static void statCountersCopy(StatCounters * dest, const StatCounters * orig);
 static double statMedianSvc(int, int);
-static void statStoreEntry(StoreEntry * s, StoreEntry * e);
+static void statStoreEntry(MemBuf * mb, StoreEntry * e);
 static double statCPUUsage(int minutes);
 static OBJH stat_io_get;
 static OBJH stat_objects_get;
@@ -79,13 +79,19 @@ static OBJH statCountersHistograms;
 static OBJH statClientRequests;
 
 #ifdef XMALLOC_STATISTICS
-static void info_get_mallstat(int, int, StoreEntry *);
+static void info_get_mallstat(int, int, int, void *);
+static double xm_time;
+static double xm_deltat;
 #endif
 
 StatCounters CountHist[N_COUNT_HIST];
 static int NCountHist = 0;
 static StatCounters CountHourHist[N_COUNT_HOUR_HIST];
 static int NCountHourHist = 0;
+CBDATA_TYPE(StatObjectsState);
+
+extern unsigned int mem_pool_alloc_calls;
+extern unsigned int mem_pool_free_calls;
 
 static void
 statUtilization(StoreEntry * e)
@@ -248,54 +254,55 @@ describeTimestamps(const StoreEntry * entry)
 }
 
 static void
-statStoreEntry(StoreEntry * s, StoreEntry * e)
+statStoreEntry(MemBuf * mb, StoreEntry * e)
 {
     MemObject *mem = e->mem_obj;
     int i;
     struct _store_client *sc;
-    storeAppendPrintf(s, "KEY %s\n", storeKeyText(e->key));
+    dlink_node *node;
+    memBufPrintf(mb, "KEY %s\n", storeKeyText(e->hash.key));
     if (mem)
-	storeAppendPrintf(s, "\t%s %s\n",
+	memBufPrintf(mb, "\t%s %s\n",
 	    RequestMethodStr[mem->method], mem->log_url);
-    storeAppendPrintf(s, "\t%s\n", describeStatuses(e));
-    storeAppendPrintf(s, "\t%s\n", storeEntryFlags(e));
-    storeAppendPrintf(s, "\t%s\n", describeTimestamps(e));
-    storeAppendPrintf(s, "\t%d locks, %d clients, %d refs\n",
+    memBufPrintf(mb, "\t%s\n", describeStatuses(e));
+    memBufPrintf(mb, "\t%s\n", storeEntryFlags(e));
+    memBufPrintf(mb, "\t%s\n", describeTimestamps(e));
+    memBufPrintf(mb, "\t%d locks, %d clients, %d refs\n",
 	(int) e->lock_count,
 	storePendingNClients(e),
 	(int) e->refcount);
-    storeAppendPrintf(s, "\tSwap File %#08X\n",
-	e->swap_file_number);
+    memBufPrintf(mb, "\tSwap Dir %d, File %#08X\n",
+	e->swap_dirn, e->swap_filen);
     if (mem != NULL) {
-	storeAppendPrintf(s, "\tinmem_lo: %d\n", (int) mem->inmem_lo);
-	storeAppendPrintf(s, "\tinmem_hi: %d\n", (int) mem->inmem_hi);
-	storeAppendPrintf(s, "\tswapout: %d bytes done, %d queued, FD %d\n",
-	    (int) mem->swapout.done_offset,
-	    (int) mem->swapout.queue_offset,
-	    mem->swapout.fd);
-	for (i = 0, sc = &mem->clients[i]; sc != NULL; sc = sc->next, i++) {
+	memBufPrintf(mb, "\tinmem_lo: %d\n", (int) mem->inmem_lo);
+	memBufPrintf(mb, "\tinmem_hi: %d\n", (int) mem->inmem_hi);
+	memBufPrintf(mb, "\tswapout: %d bytes queued\n",
+	    (int) mem->swapout.queue_offset);
+	if (mem->swapout.sio)
+	    memBufPrintf(mb, "\tswapout: %d bytes written\n",
+		(int) storeOffset(mem->swapout.sio));
+	for (i = 0, node = mem->clients.head; node; node = node->next, i++) {
+	    sc = (store_client *) node->data;
 	    if (sc->callback_data == NULL)
 		continue;
-	    storeAppendPrintf(s, "\tClient #%d, %p\n", i, sc->callback_data);
-	    storeAppendPrintf(s, "\t\tcopy_offset: %d\n",
+	    memBufPrintf(mb, "\tClient #%d, %p\n", i, sc->callback_data);
+	    memBufPrintf(mb, "\t\tcopy_offset: %d\n",
 		(int) sc->copy_offset);
-	    storeAppendPrintf(s, "\t\tseen_offset: %d\n",
+	    memBufPrintf(mb, "\t\tseen_offset: %d\n",
 		(int) sc->seen_offset);
-	    storeAppendPrintf(s, "\t\tcopy_size: %d\n",
+	    memBufPrintf(mb, "\t\tcopy_size: %d\n",
 		(int) sc->copy_size);
-	    storeAppendPrintf(s, "\t\tswapin_fd: %d\n",
-		(int) sc->swapin_fd);
-	    storeAppendPrintf(s, "\t\tflags:");
+	    memBufPrintf(mb, "\t\tflags:");
 	    if (sc->flags.disk_io_pending)
-		storeAppendPrintf(s, " disk_io_pending");
+		memBufPrintf(mb, " disk_io_pending");
 	    if (sc->flags.store_copying)
-		storeAppendPrintf(s, " store_copying");
+		memBufPrintf(mb, " store_copying");
 	    if (sc->flags.copy_event_pending)
-		storeAppendPrintf(s, " copy_event_pending");
-	    storeAppendPrintf(s, "\n");
+		memBufPrintf(mb, " copy_event_pending");
+	    memBufPrintf(mb, "\n");
 	}
     }
-    storeAppendPrintf(s, "\n");
+    memBufPrintf(mb, "\n");
 }
 
 /* process objects list */
@@ -319,29 +326,33 @@ statObjects(void *data)
 	eventAdd("statObjects", statObjects, state, 0.1, 1);
 	return;
     }
-    storeBuffer(state->sentry);
     debug(49, 3) ("statObjects: Bucket #%d\n", state->bucket);
     link_next = hash_get_bucket(store_table, state->bucket);
-    while (NULL != (link_ptr = link_next)) {
-	link_next = link_ptr->next;
-	e = (StoreEntry *) link_ptr;
-	if (state->filter && 0 == state->filter(e))
-	    continue;
-	statStoreEntry(state->sentry, e);
+    if (link_next) {
+	MemBuf mb;
+	memBufDefInit(&mb);
+	while (NULL != (link_ptr = link_next)) {
+	    link_next = link_ptr->next;
+	    e = (StoreEntry *) link_ptr;
+	    if (state->filter && 0 == state->filter(e))
+		continue;
+	    statStoreEntry(&mb, e);
+	}
+	storeAppend(state->sentry, mb.buf, mb.size);
+	memBufClean(&mb);
     }
     state->bucket++;
     eventAdd("statObjects", statObjects, state, 0.0, 1);
-    storeBufferFlush(state->sentry);
 }
 
 static void
 statObjectsStart(StoreEntry * sentry, STOBJFLT * filter)
 {
-    StatObjectsState *state = xcalloc(1, sizeof(*state));
+    StatObjectsState *state;
+    state = cbdataAlloc(StatObjectsState);
     state->sentry = sentry;
     state->filter = filter;
     storeLockObject(sentry);
-    cbdataAdd(state, cbdataXfree, 0);
     eventAdd("statObjects", statObjects, state, 0.0, 1);
 }
 
@@ -369,8 +380,8 @@ statObjectsOpenfdFilter(const StoreEntry * e)
 {
     if (e->mem_obj == NULL)
 	return 0;
-    if (e->mem_obj->swapout.fd < 0)
-	return 0;;
+    if (e->mem_obj->swapout.sio == NULL)
+	return 0;
     return 1;
 }
 
@@ -384,10 +395,11 @@ statOpenfdObj(StoreEntry * sentry)
 
 #ifdef XMALLOC_STATISTICS
 static void
-info_get_mallstat(int size, int number, StoreEntry * sentry)
+info_get_mallstat(int size, int number, int oldnum, void *data)
 {
+    StoreEntry *sentry = data;
     if (number > 0)
-	storeAppendPrintf(sentry, "\t%d = %d\n", size, number);
+	storeAppendPrintf(sentry, "%d\t %d\t %d\t %.1f\n", size, number, number - oldnum, xdiv((number - oldnum), xm_deltat));
 }
 #endif
 
@@ -458,24 +470,24 @@ info_get(StoreEntry * sentry)
     storeAppendPrintf(sentry, "Connection information for %s:\n",
 	appname);
     storeAppendPrintf(sentry, "\tNumber of clients accessing cache:\t%u\n",
-	Counter.client_http.clients);
+	statCounter.client_http.clients);
     storeAppendPrintf(sentry, "\tNumber of HTTP requests received:\t%u\n",
-	Counter.client_http.requests);
+	statCounter.client_http.requests);
     storeAppendPrintf(sentry, "\tNumber of ICP messages received:\t%u\n",
-	Counter.icp.pkts_recv);
+	statCounter.icp.pkts_recv);
     storeAppendPrintf(sentry, "\tNumber of ICP messages sent:\t%u\n",
-	Counter.icp.pkts_sent);
+	statCounter.icp.pkts_sent);
     storeAppendPrintf(sentry, "\tNumber of queued ICP replies:\t%u\n",
-	Counter.icp.replies_queued);
-    storeAppendPrintf(sentry, "\tRequest failure ratio:\t%5.2f%%\n",
+	statCounter.icp.replies_queued);
+    storeAppendPrintf(sentry, "\tRequest failure ratio:\t%5.2f\n",
 	request_failure_ratio);
 
-    storeAppendPrintf(sentry, "\tHTTP requests per minute:\t%.1f\n",
-	Counter.client_http.requests / (runtime / 60.0));
-    storeAppendPrintf(sentry, "\tICP messages per minute:\t%.1f\n",
-	(Counter.icp.pkts_sent + Counter.icp.pkts_recv) / (runtime / 60.0));
+    storeAppendPrintf(sentry, "\tAverage HTTP requests per minute since start:\t%.1f\n",
+	statCounter.client_http.requests / (runtime / 60.0));
+    storeAppendPrintf(sentry, "\tAverage ICP messages per minute since start:\t%.1f\n",
+	(statCounter.icp.pkts_sent + statCounter.icp.pkts_recv) / (runtime / 60.0));
     storeAppendPrintf(sentry, "\tSelect loop called: %d times, %0.3f ms avg\n",
-	Counter.select_loops, 1000.0 * runtime / Counter.select_loops);
+	statCounter.select_loops, 1000.0 * runtime / statCounter.select_loops);
 
     storeAppendPrintf(sentry, "Cache information for %s:\n",
 	appname);
@@ -485,16 +497,20 @@ info_get(StoreEntry * sentry)
     storeAppendPrintf(sentry, "\tByte Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
 	statByteHitRatio(5),
 	statByteHitRatio(60));
+    storeAppendPrintf(sentry, "\tRequest Memory Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
+	statRequestHitMemoryRatio(5),
+	statRequestHitMemoryRatio(60));
+    storeAppendPrintf(sentry, "\tRequest Disk Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
+	statRequestHitDiskRatio(5),
+	statRequestHitDiskRatio(60));
     storeAppendPrintf(sentry, "\tStorage Swap size:\t%d KB\n",
 	store_swap_size);
     storeAppendPrintf(sentry, "\tStorage Mem size:\t%d KB\n",
 	(int) (store_mem_size >> 10));
-    storeAppendPrintf(sentry, "\tStorage LRU Expiration Age:\t%6.2f days\n",
-	(double) storeExpiredReferenceAge() / 86400.0);
     storeAppendPrintf(sentry, "\tMean Object Size:\t%0.2f KB\n",
 	n_disk_objects ? (double) store_swap_size / n_disk_objects : 0.0);
     storeAppendPrintf(sentry, "\tRequests given to unlinkd:\t%d\n",
-	Counter.unlink.requests);
+	statCounter.unlink.requests);
 
     storeAppendPrintf(sentry, "Median Service Times (seconds)  5 min    60 min:\n");
     storeAppendPrintf(sentry, "\tHTTP Requests (All):  %8.5f %8.5f\n",
@@ -530,6 +546,10 @@ info_get(StoreEntry * sentry)
 	statCPUUsage(5));
     storeAppendPrintf(sentry, "\tCPU Usage, 60 minute avg:\t%.2f%%\n",
 	statCPUUsage(60));
+#if HAVE_SBRK
+    storeAppendPrintf(sentry, "\tProcess Data Segment Size via sbrk(): %d KB\n",
+	(int) (((char *) sbrk(0) - (char *) sbrk_start) >> 10));
+#endif
     storeAppendPrintf(sentry, "\tMaximum Resident Size: %d KB\n",
 	rusage_maxrss(&rusage));
     storeAppendPrintf(sentry, "\tPage faults with physical i/o: %d\n",
@@ -561,10 +581,13 @@ info_get(StoreEntry * sentry)
 	mp.fordblks >> 10);
     t = mp.uordblks + mp.usmblks + mp.hblkhd;
     storeAppendPrintf(sentry, "\tTotal in use:          %6d KB %d%%\n",
-	t >> 10, percent(t, mp.arena));
+	t >> 10, percent(t, mp.arena + mp.hblkhd));
     t = mp.fsmblks + mp.fordblks;
     storeAppendPrintf(sentry, "\tTotal free:            %6d KB %d%%\n",
-	t >> 10, percent(t, mp.arena));
+	t >> 10, percent(t, mp.arena + mp.hblkhd));
+    t = mp.arena + mp.hblkhd;
+    storeAppendPrintf(sentry, "\tTotal size:            %6d KB\n",
+	t >> 10);
 #if HAVE_EXT_MALLINFO
     storeAppendPrintf(sentry, "\tmax size of small blocks:\t%d\n", mp.mxfast);
     storeAppendPrintf(sentry, "\tnumber of small blocks in a holding block:\t%d\n",
@@ -580,7 +603,11 @@ info_get(StoreEntry * sentry)
 #endif /* HAVE_MALLINFO */
     storeAppendPrintf(sentry, "Memory accounted for:\n");
     storeAppendPrintf(sentry, "\tTotal accounted:       %6d KB\n",
-	memTotalAllocated() >> 10);
+	statMemoryAccounted() >> 10);
+    storeAppendPrintf(sentry, "\tmemPoolAlloc calls: %u\n",
+	mem_pool_alloc_calls);
+    storeAppendPrintf(sentry, "\tmemPoolFree calls: %u\n",
+	mem_pool_free_calls);
 
     storeAppendPrintf(sentry, "File descriptor usage for %s:\n", appname);
     storeAppendPrintf(sentry, "\tMaximum number of file descriptors:   %4d\n",
@@ -605,13 +632,14 @@ info_get(StoreEntry * sentry)
 	memInUse(MEM_MEMOBJECT));
     storeAppendPrintf(sentry, "\t%6d Hot Object Cache Items\n",
 	hot_obj_count);
-    storeAppendPrintf(sentry, "\t%6d Filemap bits set\n",
-	storeDirMapBitsInUse());
     storeAppendPrintf(sentry, "\t%6d on-disk objects\n",
 	n_disk_objects);
 
 #if XMALLOC_STATISTICS
-    storeAppendPrintf(sentry, "Memory allocation statistics\n");
+    xm_deltat = current_dtime - xm_time;
+    xm_time = current_dtime;
+    storeAppendPrintf(sentry, "\nMemory allocation statistics\n");
+    storeAppendPrintf(sentry, "Allocation Size\t Alloc Count\t Alloc Delta\t Allocs/sec \n");
     malloc_statistics(info_get_mallstat, sentry);
 #endif
 }
@@ -775,8 +803,12 @@ statAvgDump(StoreEntry * sentry, int minutes, int hours)
 	: 0.0);
     x = statHistDeltaMedian(&l->select_fds_hist, &f->select_fds_hist);
     storeAppendPrintf(sentry, "median_select_fds = %f\n", x);
-    storeAppendPrintf(sentry, "swap_files_cleaned = %f/sec\n",
-	XAVG(swap_files_cleaned));
+    storeAppendPrintf(sentry, "swap.outs = %f/sec\n",
+	XAVG(swap.outs));
+    storeAppendPrintf(sentry, "swap.ins = %f/sec\n",
+	XAVG(swap.ins));
+    storeAppendPrintf(sentry, "swap.files_cleaned = %f/sec\n",
+	XAVG(swap.files_cleaned));
     storeAppendPrintf(sentry, "aborted_requests = %f/sec\n",
 	XAVG(aborted_requests));
 
@@ -812,11 +844,12 @@ statInit(void)
 {
     int i;
     debug(18, 5) ("statInit: Initializing...\n");
+    CBDATA_INIT_TYPE(StatObjectsState);
     for (i = 0; i < N_COUNT_HIST; i++)
 	statCountersInit(&CountHist[i]);
     for (i = 0; i < N_COUNT_HOUR_HIST; i++)
 	statCountersInit(&CountHourHist[i]);
-    statCountersInit(&Counter);
+    statCountersInit(&statCounter);
     eventAdd("statAvgTick", statAvgTick, NULL, (double) COUNT_INTERVAL, 1);
     cachemgrRegister("info",
 	"General Runtime Information",
@@ -876,7 +909,7 @@ statAvgTick(void *notused)
 {
     StatCounters *t = &CountHist[0];
     StatCounters *p = &CountHist[1];
-    StatCounters *c = &Counter;
+    StatCounters *c = &statCounter;
     struct rusage rusage;
     eventAdd("statAvgTick", statAvgTick, NULL, (double) COUNT_INTERVAL, 1);
     squid_getrusage(&rusage);
@@ -899,6 +932,34 @@ statAvgTick(void *notused)
 	statCountersCopy(t, c);
 	NCountHourHist++;
     }
+    if (Config.warnings.high_rptm > 0) {
+	int i = (int) statMedianSvc(20, MEDIAN_HTTP);
+	if (Config.warnings.high_rptm < i)
+	    debug(18, 0) ("WARNING: Median response time is %d milliseconds\n", i);
+    }
+    if (Config.warnings.high_pf) {
+	int i = (CountHist[0].page_faults - CountHist[1].page_faults);
+	double dt = tvSubDsec(CountHist[0].timestamp, CountHist[1].timestamp);
+	if (i > 0 && dt > 0.0) {
+	    i /= (int) dt;
+	    if (Config.warnings.high_pf < i)
+		debug(18, 0) ("WARNING: Page faults occuring at %d/sec\n", i);
+	}
+    }
+    if (Config.warnings.high_memory) {
+	int i = 0;
+#if HAVE_MSTATS && HAVE_GNUMALLOC_H
+	struct mstats ms = mstats();
+	i = ms.bytes_total;
+#elif HAVE_MALLINFO && HAVE_STRUCT_MALLINFO
+	struct mallinfo mp = mallinfo();
+	i = mp.arena;
+#elif HAVE_SBRK
+	i = (size_t) ((char *) sbrk(0) - (char *) sbrk_start);
+#endif
+	if (Config.warnings.high_memory < i)
+	    debug(18, 0) ("WARNING: Memory usage at %d MB\n", i >> 20);
+    }
 }
 
 static void
@@ -936,8 +997,9 @@ statCountersInitSpecial(StatCounters * C)
      */
     statHistEnumInit(&C->cd.on_xition_count, CacheDigestHashFuncCount);
     statHistEnumInit(&C->comm_icp_incoming, INCOMING_ICP_MAX);
+    statHistEnumInit(&C->comm_dns_incoming, INCOMING_DNS_MAX);
     statHistEnumInit(&C->comm_http_incoming, INCOMING_HTTP_MAX);
-    statHistIntInit(&C->select_fds_hist, SQUID_MAXFD);
+    statHistIntInit(&C->select_fds_hist, 256);	/* was SQUID_MAXFD, but it is way too much. It is OK to crop this statistics */
 }
 
 /* add special cases here as they arrive */
@@ -955,6 +1017,7 @@ statCountersClean(StatCounters * C)
     statHistClean(&C->dns.svc_time);
     statHistClean(&C->cd.on_xition_count);
     statHistClean(&C->comm_icp_incoming);
+    statHistClean(&C->comm_dns_incoming);
     statHistClean(&C->comm_http_incoming);
     statHistClean(&C->select_fds_hist);
 }
@@ -965,7 +1028,7 @@ statCountersCopy(StatCounters * dest, const StatCounters * orig)
 {
     assert(dest && orig);
     /* this should take care of all the fields, but "special" ones */
-    memcpy(dest, orig, sizeof(*dest));
+    xmemcpy(dest, orig, sizeof(*dest));
     /* prepare space where to copy special entries */
     statCountersInitSpecial(dest);
     /* now handle special cases */
@@ -987,7 +1050,7 @@ statCountersCopy(StatCounters * dest, const StatCounters * orig)
 static void
 statCountersHistograms(StoreEntry * sentry)
 {
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     storeAppendPrintf(sentry, "client_http.all_svc_time histogram:\n");
     statHistDump(&f->client_http.all_svc_time, sentry, NULL);
     storeAppendPrintf(sentry, "client_http.miss_svc_time histogram:\n");
@@ -1004,12 +1067,14 @@ statCountersHistograms(StoreEntry * sentry)
     statHistDump(&f->icp.reply_svc_time, sentry, NULL);
     storeAppendPrintf(sentry, "dns.svc_time histogram:\n");
     statHistDump(&f->dns.svc_time, sentry, NULL);
+    storeAppendPrintf(sentry, "select_fds_hist histogram:\n");
+    statHistDump(&f->select_fds_hist, sentry, NULL);
 }
 
 static void
 statCountersDump(StoreEntry * sentry)
 {
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     struct rusage rusage;
     squid_getrusage(&rusage);
     f->page_faults = rusage_pagefaults(&rusage);
@@ -1126,8 +1191,12 @@ statCountersDump(StoreEntry * sentry)
 	f->cputime);
     storeAppendPrintf(sentry, "wall_time = %f\n",
 	tvSubDsec(f->timestamp, current_time));
-    storeAppendPrintf(sentry, "swap_files_cleaned = %d\n",
-	f->swap_files_cleaned);
+    storeAppendPrintf(sentry, "swap.outs = %d\n",
+	f->swap.outs);
+    storeAppendPrintf(sentry, "swap.ins = %d\n",
+	f->swap.ins);
+    storeAppendPrintf(sentry, "swap.files_cleaned = %d\n",
+	f->swap.files_cleaned);
     storeAppendPrintf(sentry, "aborted_requests = %d\n",
 	f->aborted_requests);
 }
@@ -1146,7 +1215,7 @@ static void
 statPeerSelect(StoreEntry * sentry)
 {
 #if USE_CACHE_DIGESTS
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     peer *peer;
     const int tot_used = f->cd.times_used + f->icp.times_used;
 
@@ -1246,12 +1315,15 @@ statMedianSvc(int interval, int which)
 /*
  * SNMP wants ints, ick
  */
+#if UNUSED_CODE
 int
 get_median_svc(int interval, int which)
 {
     return (int) statMedianSvc(interval, which);
 }
 
+#endif
+
 StatCounters *
 snmpStatGet(int minutes)
 {
@@ -1262,7 +1334,7 @@ int
 stat5minClientRequests(void)
 {
     assert(N_COUNT_HIST > 5);
-    return Counter.client_http.requests - CountHist[5].client_http.requests;
+    return statCounter.client_http.requests - CountHist[5].client_http.requests;
 }
 
 static double
@@ -1283,15 +1355,51 @@ statRequestHitRatio(int minutes)
 	CountHist[minutes].client_http.requests);
 }
 
+extern double
+statRequestHitMemoryRatio(int minutes)
+{
+    assert(minutes < N_COUNT_HIST);
+    return dpercent(CountHist[0].client_http.mem_hits -
+	CountHist[minutes].client_http.mem_hits,
+	CountHist[0].client_http.hits -
+	CountHist[minutes].client_http.hits);
+}
+
+extern double
+statRequestHitDiskRatio(int minutes)
+{
+    assert(minutes < N_COUNT_HIST);
+    return dpercent(CountHist[0].client_http.disk_hits -
+	CountHist[minutes].client_http.disk_hits,
+	CountHist[0].client_http.hits -
+	CountHist[minutes].client_http.hits);
+}
+
 extern double
 statByteHitRatio(int minutes)
 {
     size_t s;
     size_t c;
+#if USE_CACHE_DIGESTS
+    size_t cd;
+#endif
+    /* size_t might be unsigned */
     assert(minutes < N_COUNT_HIST);
     c = CountHist[0].client_http.kbytes_out.kb - CountHist[minutes].client_http.kbytes_out.kb;
     s = CountHist[0].server.all.kbytes_in.kb - CountHist[minutes].server.all.kbytes_in.kb;
-    /* size_t might be unsigned */
+#if USE_CACHE_DIGESTS
+    /*
+     * This ugly hack is here to prevent the user from seeing a
+     * negative byte hit ratio.  When we fetch a cache digest from
+     * a neighbor, it gets treated like a cache miss because the
+     * object is consumed internally.  Thus, we subtract cache
+     * digest bytes out before calculating the byte hit ratio.
+     */
+    cd = CountHist[0].cd.kbytes_recv.kb - CountHist[minutes].cd.kbytes_recv.kb;
+    if (s < cd)
+	debug(18, 1) ("STRANGE: srv_kbytes=%d, cd_kbytes=%d\n", s, cd);
+    s -= cd;
+#endif
     if (c > s)
 	return dpercent(c - s, c);
     else
@@ -1316,8 +1424,8 @@ statClientRequests(StoreEntry * s)
 	    storeAppendPrintf(s, "\tFD %d, read %d, wrote %d\n", fd,
 		fd_table[fd].bytes_read, fd_table[fd].bytes_written);
 	    storeAppendPrintf(s, "\tFD desc: %s\n", fd_table[fd].desc);
-	    storeAppendPrintf(s, "\tin: buf %p, offset %d, size %d\n",
-		conn->in.buf, conn->in.offset, conn->in.size);
+	    storeAppendPrintf(s, "\tin: buf %p, offset %ld, size %ld\n",
+		conn->in.buf, (long int) conn->in.offset, (long int) conn->in.size);
 	    storeAppendPrintf(s, "\tpeer: %s:%d\n",
 		inet_ntoa(conn->peer.sin_addr),
 		ntohs(conn->peer.sin_port));
@@ -1326,22 +1434,25 @@ statClientRequests(StoreEntry * s)
 		ntohs(conn->me.sin_port));
 	    storeAppendPrintf(s, "\tnrequests: %d\n",
 		conn->nrequests);
-	    storeAppendPrintf(s, "\tpersistent: %d\n",
-		conn->persistent);
-	    storeAppendPrintf(s, "\tdefer: n %d, until %d\n",
-		conn->defer.n, conn->defer.until);
+	    storeAppendPrintf(s, "\tdefer: n %d, until %ld\n",
+		conn->defer.n, (long int) conn->defer.until);
 	}
 	storeAppendPrintf(s, "uri %s\n", http->uri);
-	storeAppendPrintf(s, "out.offset %d, out.size %d\n",
-	    http->out.offset, http->out.size);
-	storeAppendPrintf(s, "req_sz %d\n", http->req_sz);
+	storeAppendPrintf(s, "log_type %s\n", log_tags[http->log_type]);
+	storeAppendPrintf(s, "out.offset %ld, out.size %lu\n",
+	    (long int) http->out.offset, (unsigned long int) http->out.size);
+	storeAppendPrintf(s, "req_sz %ld\n", (long int) http->req_sz);
 	e = http->entry;
-	storeAppendPrintf(s, "entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
+	storeAppendPrintf(s, "entry %p/%s\n", e, e ? storeKeyText(e->hash.key) : "N/A");
 	e = http->old_entry;
-	storeAppendPrintf(s, "old_entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
-	storeAppendPrintf(s, "start %d.%06d (%f seconds ago)\n", http->start.tv_sec,
-	    http->start.tv_usec,
+	storeAppendPrintf(s, "old_entry %p/%s\n", e, e ? storeKeyText(e->hash.key) : "N/A");
+	storeAppendPrintf(s, "start %ld.%06d (%f seconds ago)\n",
+	    (long int) http->start.tv_sec,
+	    (int) http->start.tv_usec,
 	    tvSubDsec(http->start, current_time));
+#if DELAY_POOLS
+	storeAppendPrintf(s, "delay_pool %d\n", delayClient(http) >> 16);
+#endif
 	storeAppendPrintf(s, "\n");
     }
 }
@@ -1429,3 +1540,9 @@ statGraphDump(StoreEntry * e)
 }
 
 #endif /* STAT_GRAPHS */
+
+int
+statMemoryAccounted(void)
+{
+    return memTotalAllocated();
+}
